@@ -134,6 +134,11 @@
 /* Define to the version of the distribution.  */
 #undef VERSION
 
+/* Define if you have a fallback MDA installed */
+#undef FALLBACK_MDA
+
+/* Define if `sys_siglist' is declared by <signal.h>.  */
+#undef SYS_SIGLIST_DECLARED
 
 
 /* Leave that blank line there!!  Autoheader needs it.
