@@ -60,7 +60,7 @@ static int etrn_getrange(int sock, struct query *ctl, const char *id,
      * By default, the hostlist has a single entry, the fetchmail host's
      * canonical DNS name.
      */
-    for (qnp = ctl->smtphunt; qnp; qnp = qnp->next)
+    for (qnp = ctl->domainlist; qnp; qnp = qnp->next)
     {
 	/* ship the actual poll and get the response */
 	gen_send(sock, "ETRN %s", qnp->id);
@@ -130,7 +130,6 @@ const static struct method etrn =
     FALSE,		/* this is not a tagged protocol */
     FALSE,		/* this does not use a message delimiter */
     etrn_ok,		/* parse command response */
-    NULL,		/* no password canonicalization */
     NULL,		/* no need to get authentication */
     etrn_getrange,	/* initialize message sending */
     NULL,		/* we cannot get a list of sizes */
