@@ -255,6 +255,7 @@ struct query
 
     /* per-forwarding-target data */
     struct idlist *smtphunt;	/* list of SMTP hosts to try forwarding to */
+    struct idlist *domainlist;	/* domainlist to fetch from */
     char *smtpaddress;		/* address to force in RCPT TO */ 
     char *smtpname;             /* full RCPT TO name, including domain */
     struct idlist *antispam;	/* list of listener's antispam response */
@@ -298,8 +299,8 @@ struct query
     const char *destaddr;	/* destination host for this query */
     int errcount;		/* count transient errors in last pass */
     int authfailcount;		/* count of authorization failures */
-	int wehaveauthed;   /* We have managed to logon at least once! */
-	int wehavesentauthnote;   /* We have sent an authorization failure note */
+    int wehaveauthed;		/* We've managed to logon at least once! */
+    int wehavesentauthnote;	/* We've sent an authorization failure note */
     int wedged;			/* wedged by auth failures or timeouts? */
     char *smtphost;		/* actual SMTP host we connected to */
     int smtp_socket;		/* socket descriptor for SMTP connection */
@@ -321,6 +322,7 @@ struct msgblk			/* message header parsed for open_sink() */
     char   		*headers;	/* raw message headers */
     struct idlist	*recipients;	/* addressees */
     char		return_path[HOSTLEN + USERNAMELEN + 4]; 
+    int			msglen;
     int			reallen;
 };
 
@@ -403,22 +405,34 @@ void report_complete ();
 void report_at_line ();
 #endif
 
-/* driver.c: transaction support */
+/* driver.c -- main driver loop */
 void set_timeout(int);
+int do_protocol(struct query *, const struct method *);
+
+/* transact.c: transaction support */
+void init_transact(const struct method *);
+int readheaders(int sock,
+		       long fetchlen,
+		       long reallen,
+		       struct query *ctl,
+		int num);
+int readbody(int sock, struct query *ctl, flag forward, int len);
 #if defined(HAVE_STDARG_H)
-void gen_send (int sock, const char *, ... );
+void gen_send(int sock, const char *, ... );
 int gen_recv(int sock, char *buf, int size);
-int gen_transact (int sock, const char *, ... );
+int gen_transact(int sock, const char *, ... );
 #else
-void gen_send ();
+void gen_send();
 int gen_recv();
-int gen_transact ();
+int gen_transact();
 #endif
+extern struct msgblk msgblk;
 
 /* lock.c: concurrency locking */
 void lock_setup(void), lock_assert(void);
 void lock_or_die(void), lock_release(void);
 int lock_state(void);
+void lock_dispose(void);
 
 /* use these to track what was happening when the nonresponse timer fired */
 #define GENERAL_WAIT	0	/* unknown wait type */
@@ -448,6 +462,7 @@ extern int mytimeout;
 int interruptible_idle(int interval);
 
 /* sink.c: forwarding */
+int smtp_open(struct query *);
 int stuffline(struct query *, char *);
 int open_sink(struct query*, struct msgblk *, int*, int*);
 void release_sink(struct query *);
@@ -535,14 +550,14 @@ static char *dummy = gettext_noop("alloca failed");
 #endif
 
 /* protocol driver and methods */
-int do_protocol(struct query *, const struct method *);
 int doPOP2 (struct query *); 
 int doPOP3 (struct query *);
 int doIMAP (struct query *);
 int doETRN (struct query *);
+int doODMR (struct query *);
 
 /* authentication functions */
-int do_cram_md5(int sock, char *command, struct query *ctl);
+int do_cram_md5(int sock, char *command, struct query *ctl, char *strip);
 int do_rfc1731(int sock, char *command, char *truename);
 int do_gssauth(int sock, char *command, char *hostname, char *username);
 int do_otp(int sock, char *command, struct query *ctl);
