@@ -14,23 +14,35 @@
 #include  <stdio.h>
 #include  <errno.h>
 #include  <string.h>
+#include  <signal.h>
+#include  <time.h>
+#ifdef HAVE_MEMORY_H
+#include  <memory.h>
+#endif /* HAVE_MEMORY_H */
 #if defined(STDC_HEADERS)
 #include  <stdlib.h>
 #endif
 #if defined(HAVE_UNISTD_H)
-#include <unistd.h>
+#include  <unistd.h>
 #endif
 #if defined(HAVE_STDARG_H)
 #include  <stdarg.h>
 #else
 #include  <varargs.h>
 #endif
+#include  <ctype.h>
+#include  <time.h>
 
 #include  "fetchmail.h"
 #include  "socket.h"
 #include  "smtp.h"
 #include  "i18n.h"
 
+/* BSD portability hack...I know, this is an ugly place to put it */
+#if !defined(SIGCHLD) && defined(SIGCLD)
+#define SIGCHLD	SIGCLD
+#endif
+
 /* makes the open_sink()/close_sink() pair non-reentrant */
 static int lmtp_responses;
 
@@ -115,6 +127,10 @@ int smtp_open(struct query *ctl)
 					     ctl->server.plugout)) == -1)
 		continue;
 
+           /* return immediately for ODMR */
+           if (ctl->server.protocol == P_ODMR)
+               return(ctl->smtp_socket); /* success */
+
 	    /* are we doing SMTP or LMTP? */
 	    SMTP_setmode(ctl->listener);
 
@@ -164,6 +180,11 @@ int smtp_open(struct query *ctl)
 
 /* these are shared by open_sink and stuffline */
 static FILE *sinkfp;
+#ifndef HAVE_SIGACTION
+static RETSIGTYPE (*sigchld)(int);
+#else
+static struct sigaction sa_old;
+#endif /* HAVE_SIGACTION */
 
 int stuffline(struct query *ctl, char *buf)
 /* ship a line to the given control block's output sink (SMTP server or MDA) */
@@ -491,6 +512,9 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 /* set up sinkfp to be an input sink we can ship a message to */
 {
     struct	idlist *idp;
+#ifdef HAVE_SIGACTION
+    struct      sigaction sa_new;
+#endif /* HAVE_SIGACTION */
 
     *bad_addresses = *good_addresses = 0;
 
@@ -709,6 +733,13 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	 */
 	if (!(*good_addresses)) 
 	{
+	    if (!run.postmaster[0])
+	    {
+		if (outlevel >= O_VERBOSE)
+		    report(stderr, _("no address matches; no postmaster set.\n"));
+		SMTP_rset(ctl->smtp_socket);	/* required by RFC1870 */
+		return(PS_REFUSED);
+	    }
 	    if (strchr(run.postmaster, '@'))
 		strncpy(addr, run.postmaster, sizeof(addr));
 	    else
@@ -915,6 +946,20 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	    report(stderr, _("MDA open failed\n"));
 	    return(PS_IOERR);
 	}
+
+	/*
+	 * We need to disable the normal SIGCHLD handling here because 
+	 * sigchld_handler() would reap away the error status, returning
+	 * error status instead of 0 for successful completion.
+	 */
+#ifndef HAVE_SIGACTION
+	sigchld = signal(SIGCHLD, SIG_DFL);
+#else
+	memset (&sa_new, 0, sizeof sa_new);
+	sigemptyset (&sa_new.sa_mask);
+	sa_new.sa_handler = SIG_DFL;
+	sigaction (SIGCHLD, &sa_new, &sa_old);
+#endif /* HAVE_SIGACTION */
     }
 
     /*
@@ -938,6 +983,12 @@ void release_sink(struct query *ctl)
 	    pclose(sinkfp);
 	    sinkfp = (FILE *)NULL;
 	}
+#ifndef HAVE_SIGACTION
+	signal(SIGCHLD, sigchld);
+#else
+	sigaction (SIGCHLD, &sa_old, NULL);
+#endif /* HAVE_SIGACTION */
+	deal_with_sigchld();
     }
 }
 
@@ -956,10 +1007,16 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	}
 	else
 	    rc = 0;
+#ifndef HAVE_SIGACTION
+	signal(SIGCHLD, sigchld);
+#else
+	sigaction (SIGCHLD, &sa_old, NULL);
+#endif /* HAVE_SIGACTION */
+	deal_with_sigchld();
 	if (rc)
 	{
 	    report(stderr, 
-		   _("MDA exited abnormally or returned nonzero status\n"));
+		   _("MDA returned nonzero status %d\n"), rc);
 	    return(FALSE);
 	}
     }
